@@ -1,4 +1,3 @@
-#!/usr/bin/python -tt
 # Copyright (C) 2013 by The Linux Foundation and contributors
 #
 # This program is free software: you can redistribute it and/or modify
@@ -17,14 +16,16 @@
 import os
 import sys
 
+import time
 import json
 import fnmatch
 
 import logging
 
 from fcntl import lockf, LOCK_EX, LOCK_UN, LOCK_NB
+from StringIO import StringIO
 
-VERSION = '0.3.2'
+VERSION = '0.3.5'
 MANIFEST_LOCKH = None
 REPO_LOCKH = {}
 
@@ -53,6 +54,21 @@ def unlock_repo(fullpath):
         REPO_LOCKH[fullpath].close()
         del REPO_LOCKH[fullpath]
 
+def is_bare_git_repo(path):
+    """
+    Return True if path (which is already verified to be a directory)
+    sufficiently resembles a base git repo (good enough to fool git
+    itself).
+    """
+    logger.debug('Checking if %s is a git repository' % path)
+    if (os.path.isdir(os.path.join(path, 'objects')) and
+            os.path.isdir(os.path.join(path, 'refs')) and
+            os.path.isfile(os.path.join(path, 'HEAD'))):
+        return True
+
+    logger.debug('Skipping %s: not a git repository' % path)
+    return False
+
 def find_all_gitdirs(toplevel, ignore=[]):
     logger.info('Finding bare git repos in %s' % toplevel)
     logger.debug('Ignore list: %s' % ' '.join(ignore))
@@ -70,7 +86,7 @@ def find_all_gitdirs(toplevel, ignore=[]):
                     torm.append(name)
                     ignored = True
                     break
-            if not ignored and name.find('.git') > 0:
+            if not ignored and is_bare_git_repo(os.path.join(root, name)):
                 logger.debug('Found %s' % os.path.join(root, name))
                 gitdirs.append(os.path.join(root, name))
                 torm.append(name)
@@ -96,7 +112,20 @@ def manifest_unlock(manifile):
         lockf(MANIFEST_LOCKH, LOCK_UN)
         MANIFEST_LOCKH.close()
 
-def read_manifest(manifile):
+def read_manifest(manifile, wait=False):
+    while True:
+        if not wait or os.path.exists(manifile):
+            break
+        logger.info('Manifest file not yet found, waiting...')
+        # Unlock the manifest so other processes aren't waiting for us
+        was_locked = False
+        if MANIFEST_LOCKH is not None:
+            was_locked = True
+            manifest_unlock(manifile)
+        time.sleep(1)
+        if was_locked:
+            manifest_lock(manifile)
+
     if not os.path.exists(manifile):
         logger.info('%s not found, assuming initial run' % manifile)
         return {}
@@ -120,30 +149,44 @@ def read_manifest(manifile):
 
     return manifest
 
-def write_manifest(manifile, manifest, mtime=None):
+def write_manifest(manifile, manifest, mtime=None, pretty=False):
     import tempfile
     import shutil
     import gzip
 
+    logger.info('Writing new %s' % manifile)
+
     (dirname, basename) = os.path.split(manifile)
     (fd, tmpfile) = tempfile.mkstemp(prefix=basename, dir=dirname)
-    logger.info('Writing new %s' % manifile)
+    fh = os.fdopen(fd, 'w', 0)
+    logger.debug('Created a temporary file in %s' % tmpfile)
+    logger.debug('Writing to %s' % tmpfile)
     try:
         if manifile.find('.gz') > 0:
-            fh = gzip.open(tmpfile, 'wb')
+            gfh = gzip.GzipFile(fileobj=fh, mode='wb')
+            if pretty:
+                json.dump(manifest, gfh, indent=2, sort_keys=True)
+            else:
+                json.dump(manifest, gfh)
+            gfh.close()
         else:
-            fh = open(tmpfile, 'w')
-        # Probably should make indent configurable, but extra whitespaces
-        # don't change the size of manifest.js.gz by any appreciable amount
-        json.dump(manifest, fh, indent=2)
+            if pretty:
+                json.dump(manifest, fh, indent=2, sort_keys=True)
+            else:
+                json.dump(manifest, fh)
+
+        os.fsync(fd)
         fh.close()
         os.chmod(tmpfile, 0644)
         if mtime is not None:
+            logger.debug('Setting mtime to %s' % mtime)
             os.utime(tmpfile, (mtime, mtime))
+        logger.debug('Moving %s to %s' % (tmpfile, manifile))
         shutil.move(tmpfile, manifile)
 
     finally:
         # If something failed, don't leave these trailing around
         if os.path.exists(tmpfile):
+            logger.debug('Removing %s' % tmpfile)
             os.unlink(tmpfile)
 
